# Copyright (c) 2002 ETH Zrich, Benno Luthiger. All rights reserved.
# See also LICENSE.txt

# python
import sys, string

# zope
from OFS import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import Globals, zLOG

# Formulator
from Products.Formulator.Form import ZMIForm
from Products.Formulator.Errors import FormValidationError
from Products.Formulator import StandardFields

# Configurable
from helpers import add_and_edit

_ViewManagementScreens = 'View management screens'
_field_id = 'value'

class Configurable(SimpleItem.SimpleItem):
    """Implementation of an object which is configurable both
    manually and programmatically and can be called like a script returning
    the configured value.
    """
    
    meta_type = 'Configurable Object'
    
    security = ClassSecurityInfo()
    
    manage_options = (
        {'label':'Edit', 'action':'manage_editForm'},
        {'label':'Configurate', 'action':'manage_metaForm'},
        {'label':'Test', 'action':'manage_test'},
        ) + SimpleItem.SimpleItem.manage_options

    security.declareProtected(_ViewManagementScreens, 'manage_editForm')
    manage_editForm = PageTemplateFile(
        'www/ConfigurableEdit', globals(),  __name__='manage_editForm')

    security.declareProtected(_ViewManagementScreens, 'manage_metaForm')
    manage_metaForm = PageTemplateFile(
        'www/ConfigurableConfigurate', globals(),  __name__='manage_metaForm')

    security.declareProtected(_ViewManagementScreens, 'manage_test')
    def manage_test(self):
        """Displays the test screen, i.e. calls the Configurable's value."""
        return self._value

    security.declareProtected(_ViewManagementScreens, 'manage_main')
    manage_main = manage_editForm
    
    def __init__(self, id, title):
        self.id = id
        self.title = title
        self._name = 'Property'
        self._value = None
        self._comment = ''
        edit_form = ZMIForm('edit', 'Edit')
        
        default = StandardFields.IntegerField(
            _field_id,
            title=self._name,
            required=1,
            display_width=20)
        edit_form._setObject(default.id, default)
        self.edit_form = edit_form
    
    security.declareProtected(_ViewManagementScreens, 'edit_form')

    security.declareProtected(_ViewManagementScreens, 'value')
    def value(self):
        """Returns the value stored by this Configurable Object."""
        return self._value
    
    security.declareProtected(_ViewManagementScreens, 'name')
    def name(self):
        """Returns the name of the property managed by this Configurable Object."""
        return self._name
 
    security.declareProtected(_ViewManagementScreens, 'set')
    def set(self, value):
        """Sets the value to be stored by this Configurable Object."""
        self._value = value
    
    security.declareProtected(_ViewManagementScreens, 'rename')
    def rename(self, name):
        """Renames the property managed by this Configurable Object."""
        self._name = name
    
    security.declareProtected(_ViewManagementScreens, 'set_comment')
    def set_comment(self, comment):
        """Sets the comment explaining the property managed by this Configurable Object."""
        self._comment = comment

    security.declareProtected(_ViewManagementScreens, 'comment')
    def comment(self):
        """Returns the comment explaining the property managed by this Configurable Object."""
        return self._comment

    security.declareProtected(_ViewManagementScreens, 'type')
    def type(self):
        """Returns the field type to edit the property managed by this Configurable Object."""
        return getattr(self.edit_form, _field_id).meta_type
    
    def __call__(self, *args, **kw):
        """Call the Configurable.
        """
        return self._value

    security.declareProtected(_ViewManagementScreens, 'manage_edit')
    def manage_edit(self, REQUEST, redisplay=1):
        """Manage method to update the value stored by this Configurable Object."""
        try:
            result = self.edit_form.validate_all(REQUEST)
        except FormValidationError, e:
            messages = ["Validation error(s)"]
            # loop through all error texts and generate messages for it
            for error in e.errors:
                messages.append("%s: %s" % (error.field.get_value('title'),
                                            error.error_text))
            # join them all together in a big message
            message = string.join(messages, "<br>")
            # return to manage_editForm showing this failure message 
            return self.manage_editForm(self, REQUEST,
                                        manage_tabs_message=message)

        self.set(result['value'])
        if redisplay:
            return self.manage_main(manage_tabs_message="Changed settings.")
    
    security.declareProtected(_ViewManagementScreens, 'manage_type')
    def manage_type(self, REQUEST):
        """Manage method to update the configuration (e.g. type to edit the property)
           of this Configurable Object."""
        prec_type = self.type()
        type = REQUEST['type']
        inherit_value = (type == prec_type)
        field = self.create_field(type)
        if type == 'RadioField':
            field.values['items'] = self.create_item_list(REQUEST['items_rf'])
        elif type == 'CheckBoxField':
            pass
        elif type == 'IntegerField':
            field.values['display_width'] = REQUEST['size_if']
        elif type == 'ListField':
            if REQUEST.get('multiple',0):
                field = self.create_field('MultiListField')
            field.values['items'] = self.create_item_list(REQUEST['items_lf'])
            inherit_value = inherit_value or (prec_type == 'ListField' or prec_type == 'MultiListField')
        elif type == 'StringField':
            field.values['display_width'] = REQUEST['size_sf']
            inherit_value = inherit_value or (prec_type == 'TextAreaField')
        elif type == 'TextAreaField':
            field.values['height'] = REQUEST.get('textarea_rows', 5)
            field.values['width'] = REQUEST.get('textarea_cols', 40)
            inherit_value = inherit_value or (prec_type == 'StringField')
        
        self.set_edit_form(field, not inherit_value)
        return self.manage_main(manage_tabs_message="Updated property field.")
    
    def create_item_list(self, items):
        out_list = []
        for item in items.splitlines():
            if item.rfind('|') >= 0:
                tupel = item.split('|')
                out_list.append((tupel[0].strip(), tupel[1].strip()))
        return out_list
    
    def set_edit_form(self, field, init_value=1):
        edit_form = self.edit_form
        if hasattr(edit_form, field.id):
            setattr(edit_form, field.id, field)
        else:
            edit_form._setObject(field.id, field)
        self.edit_form = edit_form
        if init_value:
            self._value = None
    
    security.declareProtected(_ViewManagementScreens, 'get_editable_meta')
    def get_editable_meta(self):
        """Returns the editable metadata (e.g. field size or item list)
           which configures this Configurable Object."""
        field = getattr(self.edit_form, _field_id)
        type = self.type()
        out_editable = ''
        if type == 'CheckBoxField':
            pass
        elif type == 'RadioField':
            out_editable = self.create_items_editable(field.values['items'])
        elif type == 'StringField':
            out_editable = field.values['display_width']
        elif type == 'IntegerField':
            out_editable = field.values['display_width']
        elif type == 'ListField':
            out_editable = {'items':self.create_items_editable(field.values['items']),
                            'multiple':0}
        elif type == 'MultiListField':
            out_editable = {'items':self.create_items_editable(field.values['items']),
                            'multiple':1}            
        elif type == 'TextAreaField':
            out_editable = {'width':field.values['width'],
                            'height':field.values['height']}
        return out_editable
    
    def create_items_editable(self, items):
        out_editable = ''
        for item in items:
            out_editable = out_editable + '%s | %s\n' %(item[0], item[1])
        return out_editable
    
    security.declareProtected(_ViewManagementScreens, 'manage_name')
    def manage_name(self, REQUEST):
        """Manage method to update name and comment explaining the property
           managed by this Configurable Object."""
        self.rename(REQUEST['property_name'])
        self.set_comment(REQUEST['property_comment'])
        self.title = REQUEST['object_title']
        return self.manage_main(manage_tabs_message="Updated name/comment.")
    
    def create_field(self, type):
        field = getattr(StandardFields, type)(
                _field_id,
                title=self._name,
                required=1)
        return field
    
    security.declareProtected(_ViewManagementScreens, 'set_as_list')
    def set_as_list(self, items, multiple=0):
        """Method to configure programmatically this Object as ListField."""
        if multiple:
            field = self.create_field('MultiListField')
        else:
            field = self.create_field('ListField')            
        field.values['items'] = items
        self.set_edit_form(field)

    security.declareProtected(_ViewManagementScreens, 'set_as_radio')
    def set_as_radio(self, items):
        """Method to configure programmatically this Object as RadioField."""
        field = self.create_field('RadioField')
        field.values['items'] = items
        self.set_edit_form(field)

    security.declareProtected(_ViewManagementScreens, 'set_as_checkbox')
    def set_as_checkbox(self):
        """Method to configure programmatically this Object as CheckBoxField."""
        field = self.create_field('CheckBoxField')
        self.set_edit_form(field)

    security.declareProtected(_ViewManagementScreens, 'set_as_stringfield')
    def set_as_stringfield(self, field_size):
        """Method to configure programmatically this Object as StringField."""
        field = self.create_field('StringField')
        field.values['display_width'] = field_size
        self.set_edit_form(field)

    security.declareProtected(_ViewManagementScreens, 'set_as_integerfield')
    def set_as_integerfield(self, field_size):
        """Method to configure programmatically this Object as IntegerField."""
        field = self.create_field('IntegerField')
        field.values['display_width'] = field_size
        self.set_edit_form(field)
    
    security.declareProtected(_ViewManagementScreens, 'set_as_textarea')
    def set_as_textarea(self, columns, rows):
        """Method to configure programmatically this Object as TextArea."""
        field = self.create_field('TextAreaField')
        field.values['display_width'] = columns
        field.values['display_height'] = rows
        self.set_edit_form(field)
    
Globals.InitializeClass(Configurable)

manage_addConfigurableForm = PageTemplateFile(
    "www/ConfigurableAdd", globals(),
    __name__='manage_addConfigurableForm')

def manage_addConfigurable(self, id, title='', REQUEST=None):
    """Add an instance of Configurable."""
    object = Configurable(id, title)
    self._setObject(id, object)
    object = getattr(self, id)
    add_and_edit(self, id, REQUEST)
    return ''

